@@ -31,25 +31,38 @@ double log_R1_E1(int& k, double& a1);
 void dBvZINB_Expt(int &x, int &y, int &freq, double &a0, double &a1, double &a2,
                   double &b1, double &b2, double &p1, double &p2, 
                   double &p3, double &p4,
-                  Rcpp::NumericVector &expt, Rcpp::NumericVector &s_i, Rcpp::NumericVector &info, int se);
+                  Rcpp::NumericVector &expt, Rcpp::NumericVector &s_i, Rcpp::NumericVector &info, 
+                  int se, int bnb);
 void dBvZINB_Expt_vec(Rcpp::IntegerVector &xvec, Rcpp::IntegerVector &yvec, 
                       Rcpp::IntegerVector &freq, 
                       int &n, double &a0, double &a1, double &a2,
                       double &b1, double &b2, double &p1, double &p2, 
                       double &p3, double &p4,
-                      Rcpp::NumericVector &expt, Rcpp::NumericVector &s_i, Rcpp::NumericVector &info, int se);
+                      Rcpp::NumericVector &expt, Rcpp::NumericVector &s_i, Rcpp::NumericVector &info,
+                      int se, int bnb);
+
+void dBvZINB_Expt_direct(int &x, int &y, int &freq, double &a0, double &a1, double &a2,
+                         double &b1, double &b2, double &p1, double &p2, 
+                         double &p3, double &p4,
+                         Rcpp::NumericVector &l_sum, Rcpp::NumericVector &s_i, 
+                         Rcpp::NumericVector &info);
+void dBvZINB_Expt_direct_vec(Rcpp::IntegerVector &xvec, Rcpp::IntegerVector &yvec, 
+                             Rcpp::IntegerVector &freq, 
+                             int &n, double &a0, double &a1, double &a2,
+                             double &b1, double &b2, double &p1, double &p2, 
+                             double &p3, double &p4,
+                             Rcpp::NumericVector &l_sum, Rcpp::NumericVector &s_i, 
+                             Rcpp::NumericVector &info);
 
 double inv_digamma(double x, double y);
-void inv_digamma_vec(double lb[1], Rcpp::NumericVector &expt, double a[3], 
+void inv_digamma_vec(double lb[1], Rcpp::NumericVector &expt, Rcpp::NumericVector &a, 
                      double idgam[3]);
-double hfunc(double lb[1], Rcpp::NumericVector &expt, double a[3], 
+double hfunc(double lb[1], Rcpp::NumericVector &expt, Rcpp::NumericVector &a, 
                   double idgam[3]);
-void opt_lb(double lb[1], Rcpp::NumericVector &expt, double a[3], 
+void opt_lb(double lb[1], Rcpp::NumericVector &expt, Rcpp::NumericVector &a, 
             double idgam[3]);
 
-void em(Rcpp::NumericVector& param2, Rcpp::IntegerVector &xvec, Rcpp::IntegerVector &yvec, 
-        Rcpp::IntegerVector &freq, int &n, Rcpp::NumericVector &expt, Rcpp::NumericVector &info,
-        int &se, Rcpp::IntegerVector &iter, int &maxiter, double &tol, int showFlag, 
-        Rcpp::IntegerVector &nonconv, Rcpp::NumericVector trajectory);
+Rcpp::List em(Rcpp::NumericVector& param2, Rcpp::IntegerVector &xvec, Rcpp::IntegerVector &yvec, 
+        Rcpp::IntegerVector &freq, int &n, int &se, int &maxiter, double &tol, int showFlag, int bnb);
 
 #endif
